CREATE DATABASE '127.0.0.1:c:\world.GDB' PAGE_SIZE 4096 DEFAULT CHARACTER SET UNICODE_FSS;
SET NAMES UNICODE_FSS;

CREATE DOMAIN str255 VARCHAR(255) CHARACTER SET UNICODE_FSS DEFAULT NULL;
CREATE DOMAIN str50 VARCHAR(50) CHARACTER SET UNICODE_FSS DEFAULT NULL;
CREATE DOMAIN str8 VARCHAR(8) CHARACTER SET UNICODE_FSS DEFAULT NULL;
CREATE DOMAIN str4 VARCHAR(4) CHARACTER SET UNICODE_FSS DEFAULT NULL;
CREATE DOMAIN FullDate TIMESTAMP;
CREATE DOMAIN LongInt INTEGER DEFAULT 0;
CREATE DOMAIN Boolean SMALLINT DEFAULT 0 NOT NULL CHECK (VALUE IN (0, 1));
CREATE DOMAIN GEOCOORD DOUBLE PRECISION NOT NULL;

CREATE GENERATOR COUNTRIES_GEN;
CREATE GENERATOR REGIONS_GEN;
CREATE GENERATOR CITY_GEN;

/*******************************************************/

CREATE TABLE COUNTRIES  (
       COUNTRIKEY     Longint NOT NULL,
       ID      str4,
       TRANSLITNAME str255,
       NAME		str255,
       LINK   str255
);

CREATE UNIQUE INDEX XPKCOUNTRIES ON COUNTRIES
(
       COUNTRIKEY
);


ALTER TABLE COUNTRIES ADD PRIMARY KEY (COUNTRIKEY);

SET TERM !!;

CREATE PROCEDURE ADD_COUNTRI (
    ID VARCHAR(4),
    TRANSLITNAME VARCHAR(255),
    NAME VARCHAR(255),
    LINK VARCHAR(255)
    )
RETURNS (OBJECTKEY INTEGER)
AS
DECLARE VARIABLE cnt INTEGER;
DECLARE VARIABLE cnt1 INTEGER;
DECLARE VARIABLE objkey INTEGER;
BEGIN
  SELECT COUNTRIKEY FROM COUNTRIES WHERE (ID=:ID) OR (TRANSLITNAME=:TRANSLITNAME)
         INTO :cnt;
  OBJECTKEY=0;
  IF (:cnt is null) THEN
  BEGIN
     objkey=GEN_ID(COUNTRIES_GEN,1);

     INSERT INTO COUNTRIES (COUNTRIKEY, ID, TRANSLITNAME ,NAME, LINK)
           VALUES (:objkey, :ID, :TRANSLITNAME, :NAME, :LINK);

     OBJECTKEY=:objkey;
  END
  SUSPEND;

END !!

SET TERM ; !!
COMMIT;

/**********************************************************/


CREATE TABLE REGIONS  (
       REGIONKEY     Longint NOT NULL,
       COUNTRI     Longint,
       TRANSLITNAME str255,
       NAME		str255,
       LINK   str255
);

CREATE UNIQUE INDEX XPKREGIONS ON REGIONS
(
       REGIONKEY
);


ALTER TABLE REGIONS ADD PRIMARY KEY (REGIONKEY);

SET TERM !!;

CREATE PROCEDURE ADD_REGION (
    COUNTRI INTEGER,
    TRANSLITNAME VARCHAR(255),
    NAME VARCHAR(255),
    LINK VARCHAR(255)
     )
RETURNS (OBJECTKEY INTEGER)
AS
DECLARE VARIABLE cnt INTEGER;
DECLARE VARIABLE cnt1 INTEGER;
DECLARE VARIABLE objkey INTEGER;
BEGIN
  SELECT COUNTRIKEY FROM COUNTRIES WHERE COUNTRIKEY=:COUNTRI
         INTO :cnt;
  SELECT REGIONKEY FROM REGIONS WHERE (TRANSLITNAME=:TRANSLITNAME)and(NAME=:NAME)and(COUNTRI=:COUNTRI)
         INTO :cnt1;
  OBJECTKEY=0;
  IF ((:cnt is not null)and(:cnt1 is null)) THEN
  BEGIN
     objkey=GEN_ID(REGIONS_GEN,1);

     INSERT INTO REGIONS (REGIONKEY, COUNTRI, TRANSLITNAME ,NAME, LINK)
           VALUES (:objkey, :COUNTRI, :TRANSLITNAME, :NAME, :LINK);

     OBJECTKEY=:objkey;
  END
  SUSPEND;

END !!

SET TERM ; !!
COMMIT;


/**********************************************************/

CREATE TABLE CITY  (
       CITYKEY     Longint NOT NULL,
       REGION      Longint,
       TRANSLITNAME str255,
       NAME		str255,
       LINK   str255,
       LAT GEOCOORD,
       LON GEOCOORD,
       ALT GEOCOORD,
       POPULATION Integer
);

CREATE UNIQUE INDEX XPKCITY ON CITY
(
       CITYKEY
);

CREATE  INDEX XPKCITY_REGION ON CITY REGION;
CREATE  INDEX XPKCITY_LAT ON CITY  LAT;
CREATE  INDEX XPKCITY_LON ON CITY  LON;

ALTER TABLE CITY ADD PRIMARY KEY (CITYKEY);

SET TERM !!;

CREATE PROCEDURE ADD_CITY_TEXT (
    TRANSLITNAME VARCHAR(255),
    COUNTRI VARCHAR(255),
    REGION VARCHAR(255),
    LAT DOUBLE PRECISION,
    LON DOUBLE PRECISION,
    ALT DOUBLE PRECISION,
    POPULATION INTEGER,
    LINK VARCHAR(255)
     )
RETURNS (OBJECTKEY INTEGER)
AS
DECLARE VARIABLE cnt INTEGER;
DECLARE VARIABLE cnt1 INTEGER;
DECLARE VARIABLE cnt2 INTEGER;
DECLARE VARIABLE objkey INTEGER;
BEGIN
  OBJECTKEY=-1;
  SELECT COUNTRIKEY FROM COUNTRIES WHERE TRANSLITNAME=:COUNTRI
         INTO :cnt;
  IF (:cnt is not null) THEN
  BEGIN
    SELECT REGIONKEY FROM REGIONS WHERE (TRANSLITNAME=:REGION)and(COUNTRI=:cnt)
         INTO :cnt1;
    IF (:cnt1 is null) THEN
    BEGIN
      SELECT * FROM ADD_REGION(:cnt,:REGION,' ',' ')
         INTO :cnt1;
    END

    IF (:cnt1 is not null) THEN
    BEGIN
     SELECT CITYKEY FROM CITY WHERE (REGION=:cnt1)and(TRANSLITNAME=:TRANSLITNAME)and(LAT=:LAT)and(LON=:LON)
         INTO :cnt2;
     IF (:cnt2 is null) THEN
     BEGIN
       objkey=GEN_ID(CITY_GEN,1);

       INSERT INTO CITY (CITYKEY, REGION, TRANSLITNAME ,NAME, LINK, LAT, LON, ALT, POPULATION)
           VALUES (:objkey, :cnt1, :TRANSLITNAME, '', :LINK, :LAT, :LON, :ALT, :POPULATION);

       OBJECTKEY=:objkey;
     END ELSE
     BEGIN
       OBJECTKEY=:cnt2;
     END
    END                      

  END

  SUSPEND;

END !!

SET TERM ; !!
COMMIT;